package cn.com.qmth.am.service.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.am.bean.StudentScoreVo;
import cn.com.qmth.am.entity.QuestionEntity;
import cn.com.qmth.am.entity.StudentScoreEntity;
import cn.com.qmth.am.enums.DataStatus;
import cn.com.qmth.am.service.QuestionService;
import cn.com.qmth.am.service.StmmsService;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.service.StudentService;

@Service
public class StudentServiceImpl implements StudentService {

    private static final Logger log = LoggerFactory.getLogger(StudentService.class);

    @Autowired
    private StudentService studentService;

    @Autowired
    private StmmsService stmmsService;

    @Autowired
    private StudentScoreService studentScoreService;

    @Autowired
    private QuestionService questionService;

    @Override
    public void importStudent() {
        List<QuestionEntity> qs = questionService.findToDispose();
        if (CollectionUtils.isEmpty(qs)) {
            return;
        }
        Map<String, List<QuestionEntity>> qmap = new LinkedHashMap<>();
        for (QuestionEntity q : qs) {
            String key = q.getExamId() + "-" + q.getSubjectCode();
            List<QuestionEntity> tem = qmap.get(key);
            if (tem == null) {
                tem = new ArrayList<>();
                qmap.put(key, tem);
            }
            tem.add(q);
        }
        for (List<QuestionEntity> qlist : qmap.values()) {
            List<String> students = stmmsService.getUploadStudent(qlist.get(0).getExamId(),
                    qlist.get(0).getSubjectCode());
            if (CollectionUtils.isEmpty(students)) {
                continue;
            }
            updateQuestionsStatus(qlist, DataStatus.PROCESSING);
            try {
                studentScoreService.saveByQuestion(qlist, students);
                updateQuestionsStatus(qlist, DataStatus.SUCCESS);
            } catch (Exception e) {
                log.error("同步考生出错", e);
                updateQuestionsStatus(qlist, DataStatus.FAILED);
            }
        }
    }

    private void updateQuestionsStatus(List<QuestionEntity> qlist, DataStatus st) {
        for (QuestionEntity q : qlist) {
            questionService.updateStatus(q.getId(), DataStatus.PROCESSING);
        }
    }

    @Override
    public void buildImage(StudentScoreEntity score, QuestionEntity quetion) {
        if (DataStatus.WAITING.equals(score.getAnswerStatus()) || DataStatus.FAILED.equals(score.getAnswerStatus())) {
            studentService.createSlice(score, quetion);
        }
    }

    @Override
    public void createSlice(StudentScoreEntity score, QuestionEntity q) {
        if (q == null) {
            studentScoreService.updateAnswerErr(score.getId(), "未找到试题信息");
            return;
        }
        try {
            studentScoreService.createSlice(score, q);
        } catch (Exception e) {
            if (e instanceof StatusException) {
                studentScoreService.updateAnswerErr(score.getId(), e.getMessage());
            } else {
                log.error("系统异常", e);
                studentScoreService.updateAnswerErr(score.getId(), "系统异常");
            }
        }
    }

    @Transactional
    @Override
    public void reset(Long examId, String subjectCode) {
        studentScoreService.removeBy(examId, subjectCode);
    }

    @Transactional
    @Override
    public void clear(Long examId, String subjectCode) {
        studentScoreService.removeBy(examId, subjectCode);
        questionService.removeBy(examId, subjectCode);
    }

    @Override
    public void updateMarkingScore(QuestionEntity q, StudentScoreVo score) {
        List<Double> list = stmmsService.getMarkStudent(q.getExamId(), q.getSubjectCode(), q.getMainNumber(),
                q.getSubNumber(), score.getExamNumber());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        score.setMarkingScore(list.get(0));
        studentScoreService.updateMarkingScore(score);
    }
}